//----------------------------
view.onViewAdded = function (view, child) {
	log ("onViewAdded");
	log (child);
};
view.onViewRemoved = function (view, child) {
	log ("onViewRemoved");
	log (child);
};
