// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#include "standalone/source/application.cpp"
#include "standalone/source/async.cpp"
#include "standalone/source/uidescriptionwindowcontroller.cpp"
#include "standalone/source/window.cpp"
#include "standalone/source/genericalertbox.cpp"
#include "standalone/source/shareduiresources.cpp"
#include "standalone/source/helpers/value.cpp"
