// This file is part of VSTGUI. It is subject to the license terms 
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#ifndef __vstgui__
#define __vstgui__

#include "lib/vstguibase.h"
#include "lib/cbitmap.h"
#include "lib/cbitmapfilter.h"
#include "lib/cbuttonstate.h"
#include "lib/ccolor.h"
#include "lib/cdatabrowser.h"
#include "lib/cdrawcontext.h"
#include "lib/cdrawmethods.h"
#include "lib/cdropsource.h"
#include "lib/cfileselector.h"
#include "lib/cfont.h"
#include "lib/cframe.h"
#include "lib/cgradient.h"
#include "lib/cgradientview.h"
#include "lib/cgraphicspath.h"
#include "lib/clayeredviewcontainer.h"
#include "lib/clinestyle.h"
#include "lib/coffscreencontext.h"
#include "lib/copenglview.h"
#include "lib/cpoint.h"
#include "lib/crect.h"
#include "lib/crowcolumnview.h"
#include "lib/cscrollview.h"
#include "lib/cshadowviewcontainer.h"
#include "lib/csplitview.h"
#include "lib/cstring.h"
#include "lib/ctabview.h"
#include "lib/ctooltipsupport.h"
#include "lib/cview.h"
#include "lib/cviewcontainer.h"
#include "lib/cvstguitimer.h"
#include "lib/iviewlistener.h"
#include "lib/pixelbuffer.h"
#include "lib/vstguidebug.h"

#include "lib/controls/cautoanimation.h"
#include "lib/controls/cbuttons.h"
#include "lib/controls/ccolorchooser.h"
#include "lib/controls/ccontrol.h"
#include "lib/controls/cfontchooser.h"
#include "lib/controls/cknob.h"
#include "lib/controls/cmoviebitmap.h"
#include "lib/controls/cmoviebutton.h"
#include "lib/controls/coptionmenu.h"
#include "lib/controls/cparamdisplay.h"
#include "lib/controls/cscrollbar.h"
#include "lib/controls/csearchtextedit.h"
#include "lib/controls/csegmentbutton.h"
#include "lib/controls/cslider.h"
#include "lib/controls/cspecialdigit.h"
#include "lib/controls/csplashscreen.h"
#include "lib/controls/cswitch.h"
#include "lib/controls/ctextedit.h"
#include "lib/controls/ctextlabel.h"
#include "lib/controls/cvumeter.h"
#include "lib/controls/cxypad.h"

#include "lib/animation/animations.h"
#include "lib/animation/animator.h"
#include "lib/animation/timingfunctions.h"

#endif	// __vstgui__
