// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#pragma once

namespace VSTGUI {

class UIAttributes;
class IController;
class IUIDescription;
class UIDescription;
class UIDescriptionListener;
class UIDescriptionListenerAdapter;
class IViewFactory;
class InputStream;
class OutputStream;
class IBitmapCreator;
class IBitmapCreator2;
class IContentProvider;
class MemoryContentProvider;
class InputStreamContentProvider;
class UIScripting;

} // VSTGUI
