// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#include "vstgui_standalone.cpp"
#include "standalone/source/platform/win32/win32application.cpp"
#include "standalone/source/platform/win32/win32commondirectories.cpp"
#include "standalone/source/platform/win32/win32menu.cpp"
#include "standalone/source/platform/win32/win32preference.cpp"
#include "standalone/source/platform/win32/win32window.cpp"
