// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#ifndef __vstgui_uidescription__
#define __vstgui_uidescription__

#include "lib/vstguibase.h"
#include "uidescription/uidescription.h"
#include "uidescription/uidescriptionaddonregistry.h"
#include "uidescription/uiviewcreator.h"
#include "uidescription/uiviewfactory.h"
#include "uidescription/uiattributes.h"
#include "uidescription/delegationcontroller.h"

#endif
