// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#pragma once

#include "../ccolor.h"

namespace VSTGUI {

//-----------------------------------------------------------------------------
class IPlatformGradient
{
public:
	virtual void setColorStops (const GradientColorStopMap& colorStops) = 0;
	virtual void addColorStop (const GradientColorStop& colorStop) = 0;
	virtual const GradientColorStopMap& getColorStops () const = 0;

	virtual ~IPlatformGradient () noexcept = default;
};

} // VSTGUI
