// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif

#include "vstgui.cpp"

#include "lib/platform/win32/win32taskexecutor.cpp"
#include "lib/platform/win32/win32datapackage.cpp"
#include "lib/platform/win32/win32directcomposition.cpp"
#include "lib/platform/win32/win32dragging.cpp"
#include "lib/platform/win32/win32factory.cpp"
#include "lib/platform/win32/win32frame.cpp"
#include "lib/platform/win32/win32openglview.cpp"
#include "lib/platform/win32/win32optionmenu.cpp"
#include "lib/platform/win32/win32resourcestream.cpp"
#include "lib/platform/win32/win32support.cpp"
#include "lib/platform/win32/win32textedit.cpp"
#include "lib/platform/win32/win32viewlayer.cpp"
#include "lib/platform/win32/winfileselector.cpp"
#include "lib/platform/win32/winstring.cpp"
#include "lib/platform/win32/wintimer.cpp"
#include "lib/platform/win32/direct2d/d2dbitmap.cpp"
#include "lib/platform/win32/direct2d/d2dbitmapcache.cpp"
#include "lib/platform/win32/direct2d/d2dfont.cpp"
#include "lib/platform/win32/direct2d/d2dgradient.cpp"
#include "lib/platform/win32/direct2d/d2dgraphicspath.cpp"
#include "lib/platform/win32/direct2d/d2dgraphicscontext.cpp"
