// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#include "vstgui_uidescription.h"

#include "uidescription/compresseduidescription.cpp"
#include "uidescription/cstream.cpp"
#include "uidescription/cstream_zlib.cpp"
#include "uidescription/uiattributes.cpp"
#include "uidescription/uicontentprovider.cpp"
#include "uidescription/uidescription.cpp"
#include "uidescription/uidescriptionaddonregistry.cpp"
#include "uidescription/uiviewcreator.cpp"
#include "uidescription/uiviewfactory.cpp"
#include "uidescription/uiviewswitchcontainer.cpp"

#include "uidescription/editing/uiactions.cpp"
#include "uidescription/editing/uiattributescontroller.cpp"
#include "uidescription/editing/uibitmapscontroller.cpp"
#include "uidescription/editing/uicolor.cpp"
#include "uidescription/editing/uicolorscontroller.cpp"
#include "uidescription/editing/uicolorchoosercontroller.cpp"
#include "uidescription/editing/uicolorslider.cpp"
#include "uidescription/editing/uicrosslines.cpp"
#include "uidescription/editing/uidialogcontroller.cpp"
#include "uidescription/editing/uieditcontroller.cpp"
#include "uidescription/editing/uieditmenucontroller.cpp"
#include "uidescription/editing/uieditview.cpp"
#include "uidescription/editing/uifocussettingscontroller.cpp"
#include "uidescription/editing/uifontscontroller.cpp"
#include "uidescription/editing/uigradientscontroller.cpp"
#include "uidescription/editing/uigridcontroller.cpp"
#include "uidescription/editing/uioverlayview.cpp"
#include "uidescription/editing/uiselection.cpp"
#include "uidescription/editing/uitagscontroller.cpp"
#include "uidescription/editing/uitemplatecontroller.cpp"
#include "uidescription/editing/uitemplatesettingscontroller.cpp"
#include "uidescription/editing/uiundomanager.cpp"
#include "uidescription/editing/uiviewcreatecontroller.cpp"

#include "uidescription/viewcreator/animationsplashscreencreator.cpp"
#include "uidescription/viewcreator/animknobcreator.cpp"
#include "uidescription/viewcreator/autoanimationcreator.cpp"
#include "uidescription/viewcreator/checkboxcreator.cpp"
#include "uidescription/viewcreator/controlcreator.cpp"
#include "uidescription/viewcreator/gradientviewcreator.cpp"
#include "uidescription/viewcreator/kickbuttoncreator.cpp"
#include "uidescription/viewcreator/knobcreator.cpp"
#include "uidescription/viewcreator/layeredviewcontainercreator.cpp"
#include "uidescription/viewcreator/moviebitmapcreator.cpp"
#include "uidescription/viewcreator/moviebuttoncreator.cpp"
#include "uidescription/viewcreator/multibitmapcontrolcreator.cpp"
#include "uidescription/viewcreator/multilinetextlabelcreator.cpp"
#include "uidescription/viewcreator/onoffbuttoncreator.cpp"
#include "uidescription/viewcreator/optionmenucreator.cpp"
#include "uidescription/viewcreator/paramdisplaycreator.cpp"
#include "uidescription/viewcreator/rockerswitchcreator.cpp"
#include "uidescription/viewcreator/rowcolumnviewcreator.cpp"
#include "uidescription/viewcreator/scrollviewcreator.cpp"
#include "uidescription/viewcreator/searchtexteditcreator.cpp"
#include "uidescription/viewcreator/segmentbuttoncreator.cpp"
#include "uidescription/viewcreator/shadowviewcontainercreator.cpp"
#include "uidescription/viewcreator/sliderviewcreator.cpp"
#include "uidescription/viewcreator/splitviewcreator.cpp"
#include "uidescription/viewcreator/stringlistcontrolcreator.cpp"
#include "uidescription/viewcreator/switchcreators.cpp"
#include "uidescription/viewcreator/textbuttoncreator.cpp"
#include "uidescription/viewcreator/texteditcreator.cpp"
#include "uidescription/viewcreator/textlabelcreator.cpp"
#include "uidescription/viewcreator/uiviewswitchcontainercreator.cpp"
#include "uidescription/viewcreator/viewcontainercreator.cpp"
#include "uidescription/viewcreator/viewcreator.cpp"
#include "uidescription/viewcreator/vumetercreator.cpp"
#include "uidescription/viewcreator/xypadcreator.cpp"

#include "uidescription/detail/uidesclist.cpp"
#include "uidescription/detail/uijsonpersistence.cpp"
#include "uidescription/detail/uinode.cpp"
#include "uidescription/detail/uixmlpersistence.cpp"

#include "uidescription/xmlparser.cpp" // needs to be last
