// This file is part of VSTGUI. It is subject to the license terms 
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#pragma once

#include "../include/iwindow.h"

//------------------------------------------------------------------------
namespace VSTGUI {
namespace Standalone {
namespace Detail {

//------------------------------------------------------------------------
WindowPtr makeWindow (const WindowConfiguration& config, const WindowControllerPtr& controller);

//------------------------------------------------------------------------
class IPlatformWindowAccess : public IWindow
{
public:
	virtual InterfacePtr getPlatformWindow () const = 0;
	virtual CFrame* getFrame () const = 0;
};

//------------------------------------------------------------------------
} // Detail
} // Standalone
} // VSTGUI
