// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#import "vstgui_standalone.cpp"
#import "standalone/source/platform/mac/macapplication.mm"
#import "standalone/source/platform/mac/maccommondirectories.mm"
#import "standalone/source/platform/mac/macpreference.mm"
#import "standalone/source/platform/mac/macwindow.mm"
#import "standalone/source/platform/mac/VSTGUICommand.mm"
