// This file is part of VSTGUI. It is subject to the license terms
// in the LICENSE file found in the top-level directory of this
// distribution and at http://github.com/steinbergmedia/vstgui/LICENSE

#pragma once

#include "vstgui/standalone/include/fwd.h"

//------------------------------------------------------------------------
namespace VSTGUI {
namespace Standalone {

WindowPtr makeNewDirect3DExampleWindow ();

//------------------------------------------------------------------------
} // Standalone
} // VSTGUI
